@@ -39,7 +39,7 @@
 /**
  * xsltEvalXPathPredicate:
  * @ctxt:  the XSLT transformation context
- * @str:  the XPath expression
+ * @comp:  the XPath compiled expression
  *
  * Process the expression using XPath and evaluate the result as
  * an XPath predicate
@@ -47,46 +47,36 @@
  * Returns 1 is the predicate was true, 0 otherwise
  */
 int
-xsltEvalXPathPredicate(xsltTransformContextPtr ctxt, const xmlChar *expr) {
-    int ret;
-    xmlXPathObjectPtr res, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt;
+xsltEvalXPathPredicate(xsltTransformContextPtr ctxt,
+	               xmlXPathCompExprPtr comp) {
+    int ret, position;
+    xmlXPathObjectPtr res;
 
-    xpathParserCtxt =
-	xmlXPathNewParserContext(expr, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	return(0);
+    position = ctxt->xpathCtxt->proximityPosition;
     ctxt->xpathCtxt->node = ctxt->node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
+    res = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
+    ctxt->xpathCtxt->proximityPosition = position;
     if (res != NULL) {
-	ret = xmlXPathEvaluatePredicateResult(xpathParserCtxt, res);
+	ret = xmlXPathEvalPredicate(ctxt->xpathCtxt, res);
 	xmlXPathFreeObject(res);
 #ifdef DEBUG_TEMPLATES
 	xsltGenericDebug(xsltGenericDebugContext,
-	     "xsltEvalXPathPredicate: %s returns %d\n", expr, ret);
+	     "xsltEvalXPathPredicate: returns %d\n", ret);
 #endif
     } else {
 #ifdef DEBUG_TEMPLATES
 	xsltGenericDebug(xsltGenericDebugContext,
-	     "xsltEvalXPathPredicate: %s failed\n", expr);
+	     "xsltEvalXPathPredicate: failed\n");
 #endif
 	ret = 0;
     }
-    xmlXPathFreeParserContext(xpathParserCtxt);
     return(ret);
 }
 
 /**
  * xsltEvalXPathString:
  * @ctxt:  the XSLT transformation context
- * @str:  the XPath expression
+ * @comp:  the compiled XPath expression
  *
  * Process the expression using XPath and get a string
  *
@@ -94,26 +84,15 @@ xsltEvalXPathPredicate(xsltTransformContextPtr ctxt, const xmlChar *expr) {
  *    caller.
  */
 xmlChar *
-xsltEvalXPathString(xsltTransformContextPtr ctxt, const xmlChar *expr) {
+xsltEvalXPathString(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp) {
     xmlChar *ret = NULL;
-    xmlXPathObjectPtr res, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt;
+    xmlXPathObjectPtr res;
 
-    xpathParserCtxt =
-	xmlXPathNewParserContext(expr, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	return(NULL);
     ctxt->xpathCtxt->node = ctxt->node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    xmlXPathStringFunction(xpathParserCtxt, 1);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
+    res = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
     if (res != NULL) {
+	if (res->type != XPATH_STRING)
+	    res = xmlXPathConvertString(res);
 	if (res->type == XPATH_STRING) {
             ret = res->stringval;
 	    res->stringval = NULL;
@@ -123,10 +102,9 @@ xsltEvalXPathString(xsltTransformContextPtr ctxt, const xmlChar *expr) {
 	}
 	xmlXPathFreeObject(res);
     }
-    xmlXPathFreeParserContext(xpathParserCtxt);
 #ifdef DEBUG_TEMPLATES
     xsltGenericDebug(xsltGenericDebugContext,
-	 "xsltEvalXPathString: %s returns %s\n", expr, ret);
+	 "xsltEvalXPathString: returns %s\n", ret);
 #endif
     return(ret);
 }
@@ -206,7 +184,13 @@ xsltAttrTemplateValueProcess(xsltTransformContextPtr ctxt, const xmlChar *str) {
 	    if (expr == NULL)
 		return(ret);
 	    else {
-                val = xsltEvalXPathString(ctxt, expr);
+		xmlXPathCompExprPtr comp;
+		/*
+		 * TODO: keep precompiled form around
+		 */
+		comp = xmlXPathCompile(expr);
+                val = xsltEvalXPathString(ctxt, comp);
+		xmlXPathFreeCompExpr(comp);
 		xmlFree(expr);
 		if (val != NULL) {
 		    ret = xmlStrcat(ret, val);
@@ -230,6 +214,7 @@ xsltAttrTemplateValueProcess(xsltTransformContextPtr ctxt, const xmlChar *str) {
  * @ctxt:  the XSLT transformation context
  * @node:  the stylesheet node
  * @name:  the attribute QName
+ * @ns:  the attribute namespace URI
  *
  * Evaluate a attribute value template, i.e. the attribute value can
  * contain expressions contained in curly braces ({}) and those are
@@ -240,20 +225,21 @@ xsltAttrTemplateValueProcess(xsltTransformContextPtr ctxt, const xmlChar *str) {
  */
 xmlChar *
 xsltEvalAttrValueTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
-	                  const xmlChar *name) {
+	                  const xmlChar *name, const xmlChar *ns) {
     xmlChar *ret;
     xmlChar *expr;
 
     if ((ctxt == NULL) || (node == NULL) || (name == NULL))
 	return(NULL);
 
-    expr = xmlGetNsProp(node, name, XSLT_NAMESPACE);
+    expr = xmlGetNsProp(node, name, ns);
     if (expr == NULL)
 	return(NULL);
 
     /*
-     * TODO: accelerator if there is no AttrValueTemplate in the stylesheet
-     *       return expr directly
+     * TODO: though now {} is detected ahead, it would still be good to
+     *       optimize both functions to keep the splitted value if the
+     *       attribute content and the XPath precompiled expressions around
      */
 
     ret = xsltAttrTemplateValueProcess(ctxt, expr);
@@ -270,7 +256,8 @@ xsltEvalAttrValueTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
  * xsltEvalStaticAttrValueTemplate:
  * @ctxt:  the XSLT transformation context
  * @node:  the stylesheet node
- * @name:  the attribute QName
+ * @name:  the attribute Name
+ * @name:  the attribute namespace URI
  * @found:  indicator whether the attribute is present
  *
  * Check if an attribute value template has a static value, i.e. the
@@ -281,14 +268,14 @@ xsltEvalAttrValueTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
  */
 xmlChar *
 xsltEvalStaticAttrValueTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
-	                        const xmlChar *name, int *found) {
+			const xmlChar *name, const xmlChar *ns, int *found) {
     const xmlChar *ret;
     xmlChar *expr;
 
     if ((ctxt == NULL) || (node == NULL) || (name == NULL))
 	return(NULL);
 
-    expr = xmlGetNsProp(node, name, XSLT_NAMESPACE);
+    expr = xmlGetNsProp(node, name, ns);
     if (expr == NULL) {
 	*found = 0;
 	return(NULL);
