@@ -18,19 +18,21 @@ extern "C" {
 #endif
 
 int		xsltEvalXPathPredicate		(xsltTransformContextPtr ctxt,
-						 const xmlChar *expr);
+						 xmlXPathCompExprPtr comp);
 xmlChar *	xsltEvalTemplateString		(xsltTransformContextPtr ctxt,
 						 xmlNodePtr node,
 						 xmlNodePtr parent);
 xmlChar *	xsltEvalAttrValueTemplate	(xsltTransformContextPtr ctxt,
 						 xmlNodePtr node,
-						 const xmlChar *name);
+						 const xmlChar *name,
+						 const xmlChar *ns);
 xmlChar *	xsltEvalStaticAttrValueTemplate	(xsltTransformContextPtr ctxt,
 						 xmlNodePtr node,
 						 const xmlChar *name,
+						 const xmlChar *ns,
 						 int *found);
 xmlChar *	xsltEvalXPathString		(xsltTransformContextPtr ctxt,
-						 const xmlChar *expr);
+						 xmlXPathCompExprPtr comp);
 xmlNodePtr *	xsltTemplateProcess		(xsltTransformContextPtr ctxt,
 						 xmlNodePtr node);
 xmlAttrPtr	xsltAttrListTemplateProcess	(xsltTransformContextPtr ctxt,
