@@ -224,6 +224,38 @@ xsltFreeTransformContext(xsltTransformContextPtr ctxt) {
 xmlNodePtr xsltCopyTree(xsltTransformContextPtr ctxt, xmlNodePtr node,
 			xmlNodePtr insert);
 
+/**
+ * xsltCopyProp:
+ * @ctxt:  a XSLT process context
+ * @target:  the element where the attribute will be grafted
+ * @attr:  the attribute
+ *
+ * Do a copy of an attribute
+ *
+ * Returns: a new xmlAttrPtr, or NULL in case of error.
+ */
+xmlAttrPtr
+xsltCopyProp(xsltTransformContextPtr ctxt, xmlNodePtr target,
+	     xmlAttrPtr attr) {
+    xmlAttrPtr ret = NULL;
+    xmlNsPtr ns;
+    xmlChar *val;
+
+    if (attr == NULL)
+	return(NULL);
+
+    if (attr->ns != NULL) {
+	ns = xsltGetNamespace(ctxt, attr->parent, attr->ns, target);
+    } else {
+	ns = NULL;
+    }
+    val = xmlNodeListGetString(attr->doc, attr->children, 1);
+    ret = xmlSetNsProp(target, ns, attr->name, val);
+    if (val != NULL)
+	xmlFree(val);
+    return(ret);
+}
+
 /**
  * xsltCopyPropList:
  * @ctxt:  a XSLT process context
@@ -913,7 +945,6 @@ xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	} else if ((cur->type == XML_ELEMENT_NODE) && 
 		   (cur->ns != NULL) && (cur->_private != NULL)) {
 	    xsltTransformFunction function;
-
 	    /*
 	     * Flagged as an extension element
 	     */
@@ -928,8 +959,13 @@ xsltApplyOneTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		xsltGenericDebug(xsltGenericDebugContext,
 		 "xsltApplyOneTemplate: extension construct %s\n", cur->name);
 #endif
+
+		if (cur->_private == (void *) xsltExtMarker) {
+		    cur->_private = NULL;
+		    xsltStylePreCompute(ctxt, cur);
+		}
 		ctxt->insert = insert;
-		function(ctxt, node, cur, NULL);
+		function(ctxt, node, cur, cur->_private);
 		ctxt->insert = oldInsert;
 	    }
 	    goto skip_children;
@@ -1012,6 +1048,7 @@ xsltDocumentElem(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	         xmlNodePtr inst, xsltStylePreCompPtr comp) {
     xsltStylesheetPtr style = NULL;
     int ret;
+    xmlChar *filename = NULL;
     xmlDocPtr result = NULL;
     xmlDocPtr oldOutput;
     xmlNodePtr oldInsert;
@@ -1019,8 +1056,66 @@ xsltDocumentElem(xsltTransformContextPtr ctxt, xmlNodePtr node,
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL) || (comp == NULL))
 	return;
 
-    if (comp->filename == NULL)
-	return;
+    if (comp->filename == NULL) {
+	xmlChar *base = NULL;
+	xmlChar *URL = NULL;
+	if (xmlStrEqual(inst->name, (const xmlChar *) "output")) {
+#ifdef DEBUG_EXTRA
+	    xsltGenericDebug(xsltGenericDebugContext,
+		"Found saxon:output extension\n");
+#endif
+	    filename = xsltEvalAttrValueTemplate(ctxt, inst,
+			     (const xmlChar *)"file",
+			     XSLT_SAXON_NAMESPACE);
+	} else if (xmlStrEqual(inst->name, (const xmlChar *) "write")) {
+#ifdef DEBUG_EXTRA
+	    xsltGenericDebug(xsltGenericDebugContext,
+		"Found xalan:write extension\n");
+#endif
+	    filename = xsltEvalAttrValueTemplate(ctxt, inst,
+			     (const xmlChar *)"select",
+			     XSLT_XALAN_NAMESPACE);
+	} else if (xmlStrEqual(inst->name, (const xmlChar *) "document")) {
+	    filename = xsltEvalAttrValueTemplate(ctxt, inst,
+			     (const xmlChar *)"href",
+			     XSLT_XT_NAMESPACE);
+	    if (filename == NULL) {
+#ifdef DEBUG_EXTRA
+		xsltGenericDebug(xsltGenericDebugContext,
+		    "Found xslt11:document construct\n");
+#endif
+		filename = xsltEvalAttrValueTemplate(ctxt, inst,
+				 (const xmlChar *)"href",
+				 XSLT_NAMESPACE);
+		comp->ver11 = 1;
+	    } else {
+#ifdef DEBUG_EXTRA
+		xsltGenericDebug(xsltGenericDebugContext,
+		    "Found xt:document extension\n");
+#endif
+		comp->ver11 = 0;
+	    }
+	}
+	if (filename == NULL)
+	    return;
+
+	/*
+	 * Compute output URL
+	 */
+	base = xmlNodeGetBase(inst->doc, inst);
+	URL = xmlBuildURI(filename, base);
+	if (URL == NULL) {
+	    xsltGenericError(xsltGenericErrorContext,
+		"xsltDocumentComp: URL computation failed %s\n", filename);
+	} else {
+	    xmlFree(filename);
+	    filename = URL;
+	}
+	if (base != NULL)
+	    xmlFree(base);
+    } else {
+	filename = xmlStrdup(comp->filename);
+    }
 
     oldOutput = ctxt->output;
     oldInsert = ctxt->insert;
@@ -1031,13 +1126,12 @@ xsltDocumentElem(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    "xsltDocumentElem: out of memory\n");
 	goto error;
     }
-    if (comp->ver11) {
-	/*
-	 * Version described in 1.1 draft allows full parametrization
-	 * of the output.
-	 */
-	xsltParseStylesheetOutput(style, inst);
-    }
+
+    /*
+     * Version described in 1.1 draft allows full parametrization
+     * of the output.
+     */
+    xsltParseStylesheetOutput(style, inst);
 
     /*
      * Create a new document tree and process the element template
@@ -1057,21 +1151,23 @@ xsltDocumentElem(xsltTransformContextPtr ctxt, xmlNodePtr node,
     /*
      * Save the result
      */
-    ret = xsltSaveResultToFilename((const char *) comp->filename,
-		                       result, style, 0);
+    ret = xsltSaveResultToFilename((const char *) filename,
+				   result, style, 0);
     if (ret < 0) {
 	xsltGenericError(xsltGenericErrorContext,
-	    "xsltDocumentElem: unable to save to %s\n");
+	    "xsltDocumentElem: unable to save to %s\n", filename);
     } else {
 #ifdef DEBUG_EXTRA
 	xsltGenericDebug(xsltGenericDebugContext,
-	    "Wrote %d bytes to %s\n", ret, URL);
+	    "Wrote %d bytes to %s\n", ret, , filename);
 #endif
     }
 
 error:
     ctxt->output = oldOutput;
     ctxt->insert = oldInsert;
+    if (filename != NULL)
+        xmlFree(filename);
     if (style != NULL)
 	xsltFreeStylesheet(style);
     if (result != NULL)
@@ -1100,9 +1196,7 @@ xsltSort(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
     xmlXPathObjectPtr *results = NULL;
     xmlNodeSetPtr list = NULL;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
-    xmlXPathObjectPtr res, tmp;
-    const xmlChar *start;
+    xmlXPathObjectPtr res;
     int len = 0;
     int i;
     xmlNodePtr oldNode;
@@ -1116,6 +1210,12 @@ xsltSort(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	return;
     if (comp->select == NULL)
 	return;
+    if (comp->comp == NULL) {
+	comp->comp = xmlXPathCompile(comp->select);
+	if (comp->comp == NULL)
+	    return;
+    }
+
 
     list = ctxt->nodeList;
     if ((list == NULL) || (list->nodeNr <= 1))
@@ -1127,9 +1227,6 @@ xsltSort(xsltTransformContextPtr ctxt, xmlNodePtr node,
     /* TODO: xsl:sort case-order attribute */
 
 
-    xpathParserCtxt = xmlXPathNewParserContext(comp->select, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	goto error;
     results = xmlMalloc(len * sizeof(xmlXPathObjectPtr));
     if (results == NULL) {
 	xsltGenericError(xsltGenericErrorContext,
@@ -1137,27 +1234,18 @@ xsltSort(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	goto error;
     }
 
-    start = xpathParserCtxt->cur;
     oldNode = ctxt->node;
     for (i = 0;i < len;i++) {
-	xpathParserCtxt->cur = start;
 	ctxt->xpathCtxt->contextSize = len;
 	ctxt->xpathCtxt->proximityPosition = i + 1;
 	ctxt->node = list->nodeTab[i];
 	ctxt->xpathCtxt->node = ctxt->node;
-	xmlXPathEvalExpr(xpathParserCtxt);
-	xmlXPathStringFunction(xpathParserCtxt, 1);
-	if (comp->number)
-	    xmlXPathNumberFunction(xpathParserCtxt, 1);
-	res = valuePop(xpathParserCtxt);
-	do {
-	    tmp = valuePop(xpathParserCtxt);
-	    if (tmp != NULL) {
-		xmlXPathFreeObject(tmp);
-	    }
-	} while (tmp != NULL);
-
+	res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
 	if (res != NULL) {
+	    if (res->type != XPATH_STRING)
+		res = xmlXPathConvertString(res);
+	    if (comp->number)
+		res = xmlXPathConvertNumber(res);
 	    if (comp->number) {
 		if (res->type == XPATH_NUMBER) {
 		    results[i] = res;
@@ -1186,8 +1274,6 @@ xsltSort(xsltTransformContextPtr ctxt, xmlNodePtr node,
     xsltSortFunction(list, &results[0], comp->descending, comp->number);
 
 error:
-    if (xpathParserCtxt != NULL)
-	xmlXPathFreeParserContext(xpathParserCtxt);
     if (results != NULL) {
 	for (i = 0;i < len;i++)
 	    xmlXPathFreeObject(results[i]);
@@ -1343,7 +1429,8 @@ xsltElement(xsltTransformContextPtr ctxt, xmlNodePtr node,
     oldInsert = ctxt->insert;
 
     if (comp->name == NULL) {
-	prop = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"name");
+	prop = xsltEvalAttrValueTemplate(ctxt, inst,
+		      (const xmlChar *)"name", XSLT_NAMESPACE);
 	if (prop == NULL) {
 	    xsltGenericError(xsltGenericErrorContext,
 		 "xslt:element : name is missing\n");
@@ -1362,7 +1449,7 @@ xsltElement(xsltTransformContextPtr ctxt, xmlNodePtr node,
     }
     if ((comp->ns == NULL) && (comp->has_ns)) {
 	namespace = xsltEvalAttrValueTemplate(ctxt, inst,
-		(const xmlChar *)"namespace");
+		(const xmlChar *)"namespace", XSLT_NAMESPACE);
 	if (namespace != NULL) {
 	    ns = xsltGetSpecialNamespace(ctxt, inst, namespace, prefix,
 		                         ctxt->insert);
@@ -1403,7 +1490,7 @@ xsltElement(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    xsltApplyAttributeSet(ctxt, node, inst, comp->use);
 	} else {
 	    attributes = xsltEvalAttrValueTemplate(ctxt, inst,
-			       (const xmlChar *)"use-attribute-sets");
+		       (const xmlChar *)"use-attribute-sets", XSLT_NAMESPACE);
 	    if (attributes != NULL) {
 		xsltApplyAttributeSet(ctxt, node, inst, attributes);
 		xmlFree(attributes);
@@ -1466,7 +1553,8 @@ xsltAttribute(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	return;
     }
     if (comp->name == NULL) {
-	prop = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"name");
+	prop = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"name",
+		                         XSLT_NAMESPACE);
 	if (prop == NULL) {
 	    xsltGenericError(xsltGenericErrorContext,
 		 "xslt:element : name is missing\n");
@@ -1492,7 +1580,7 @@ xsltAttribute(xsltTransformContextPtr ctxt, xmlNodePtr node,
     }
     if ((comp->ns == NULL) && (comp->has_ns)) {
 	namespace = xsltEvalAttrValueTemplate(ctxt, inst,
-		(const xmlChar *)"namespace");
+		(const xmlChar *)"namespace", XSLT_NAMESPACE);
 	if (namespace != NULL) {
 	    ns = xsltGetSpecialNamespace(ctxt, inst, namespace, prefix,
 		                         ctxt->insert);
@@ -1595,7 +1683,8 @@ xsltProcessingInstruction(xsltTransformContextPtr ctxt, xmlNodePtr node,
     if (comp->has_name == 0)
 	return;
     if (comp->name == NULL) {
-	ncname = xsltEvalAttrValueTemplate(ctxt, inst, (const xmlChar *)"name");
+	ncname = xsltEvalAttrValueTemplate(ctxt, inst,
+			    (const xmlChar *)"name", XSLT_NAMESPACE);
 	if (ncname == NULL) {
 	    xsltGenericError(xsltGenericErrorContext,
 		 "xslt:processing-instruction : name is missing\n");
@@ -1641,8 +1730,7 @@ error:
 void
 xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
-    xmlXPathObjectPtr res = NULL, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
+    xmlXPathObjectPtr res = NULL;
     xmlNodePtr copy = NULL;
     xmlNodeSetPtr list = NULL;
     int i;
@@ -1652,24 +1740,18 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     if (comp->select == NULL)
 	return;
+    if (comp->comp == NULL) {
+	comp->comp = xmlXPathCompile(comp->select);
+	if (comp->comp == NULL)
+	    return;
+    }
 #ifdef DEBUG_PROCESS
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltCopyOf: select %s\n", comp->select);
 #endif
 
-    xpathParserCtxt =
-	xmlXPathNewParserContext(comp->select, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	goto error;
     ctxt->xpathCtxt->node = node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
+    res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     if (res != NULL) {
 	if (res->type == XPATH_NODESET) {
 	    list = res->nodesetval;
@@ -1684,6 +1766,9 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 			(list->nodeTab[i]->type == XML_HTML_DOCUMENT_NODE)) {
 			xsltCopyTreeList(ctxt, list->nodeTab[i]->children,
 				         ctxt->insert);
+		    } else if (list->nodeTab[i]->type == XML_ATTRIBUTE_NODE) {
+			xsltCopyProp(ctxt, ctxt->insert, 
+				     (xmlAttrPtr) list->nodeTab[i]);
 		    } else {
 			xsltCopyTree(ctxt, list->nodeTab[i], ctxt->insert);
 		    }
@@ -1698,9 +1783,7 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    }
 	} else {
 	    /* convert to a string */
-	    valuePush(xpathParserCtxt, res);
-	    xmlXPathStringFunction(xpathParserCtxt, 1);
-	    res = valuePop(xpathParserCtxt);
+	    res = xmlXPathConvertString(res);
 	    if ((res != NULL) && (res->type == XPATH_STRING)) {
 		/* append content as text node */
 		copy = xmlNewText(res->stringval);
@@ -1708,12 +1791,6 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 		    xmlAddChild(ctxt->insert, copy);
 		}
 	    }
-	    do {
-		tmp = valuePop(xpathParserCtxt);
-		if (tmp != NULL) {
-		    xmlXPathFreeObject(tmp);
-		}
-	    } while (tmp != NULL);
 	    if (copy == NULL) {
 		xsltGenericError(xsltGenericErrorContext,
 		    "xsltDefaultProcessOneNode: text copy failed\n");
@@ -1726,11 +1803,6 @@ xsltCopyOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	}
     }
 
-error:
-    if (xpathParserCtxt != NULL) {
-	xmlXPathFreeParserContext(xpathParserCtxt);
-        xpathParserCtxt = NULL;
-    }
     if (res != NULL)
 	xmlXPathFreeObject(res);
 }
@@ -1747,8 +1819,7 @@ error:
 void
 xsltValueOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
-    xmlXPathObjectPtr res = NULL, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
+    xmlXPathObjectPtr res = NULL;
     xmlNodePtr copy = NULL;
 
     if ((ctxt == NULL) || (node == NULL) || (inst == NULL) || (comp == NULL))
@@ -1756,27 +1827,22 @@ xsltValueOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     if (comp->select == NULL)
 	return;
+    if (comp->comp == NULL) {
+	comp->comp = xmlXPathCompile(comp->select);
+	if (comp->comp == NULL)
+	    return;
+    }
 
 #ifdef DEBUG_PROCESS
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltValueOf: select %s\n", comp->select);
 #endif
 
-    xpathParserCtxt =
-	xmlXPathNewParserContext(comp->select, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	goto error;
     ctxt->xpathCtxt->node = node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    xmlXPathStringFunction(xpathParserCtxt, 1);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
+    res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     if (res != NULL) {
+	if (res->type != XPATH_STRING)
+	    res = xmlXPathConvertString(res);
 	if (res->type == XPATH_STRING) {
             copy = xmlNewText(res->stringval);
 	    if (copy != NULL) {
@@ -1795,11 +1861,6 @@ xsltValueOf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltValueOf: result %s\n", res->stringval);
 #endif
-error:
-    if (xpathParserCtxt != NULL) {
-	xmlXPathFreeParserContext(xpathParserCtxt);
-        xpathParserCtxt = NULL;
-    }
     if (res != NULL)
 	xmlXPathFreeObject(res);
 }
@@ -1930,9 +1991,8 @@ void
 xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
     xmlNodePtr cur, delete = NULL;
-    xmlXPathObjectPtr res = NULL, tmp;
+    xmlXPathObjectPtr res = NULL;
     xmlNodeSetPtr list = NULL, oldlist;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
     int i, oldProximityPosition, oldContextSize;
     const xmlChar *oldmode, *oldmodeURI;
     int have_sort=0;
@@ -1960,26 +2020,18 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
     ctxt->modeURI = comp->modeURI;
 
     if (comp->select != NULL) {
+	if (comp->comp == NULL) {
+	    comp->comp = xmlXPathCompile(comp->select);
+	    if (comp->comp == NULL)
+		goto error;
+	}
 #ifdef DEBUG_PROCESS
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltApplyTemplates: select %s\n", comp->select);
 #endif
 
-	if (ctxt->xpathCtxt == NULL) {
-	}
-	xpathParserCtxt =
-	    xmlXPathNewParserContext(comp->select, ctxt->xpathCtxt);
-	if (xpathParserCtxt == NULL)
-	    goto error;
 	ctxt->xpathCtxt->node = node;
-	xmlXPathEvalExpr(xpathParserCtxt);
-	res = valuePop(xpathParserCtxt);
-	do {
-	    tmp = valuePop(xpathParserCtxt);
-	    if (tmp != NULL) {
-		xmlXPathFreeObject(tmp);
-	    }
-	} while (tmp != NULL);
+	res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
 	if (res != NULL) {
 	    if (res->type == XPATH_NODESET) {
 		list = res->nodesetval;
@@ -2103,8 +2155,6 @@ xsltApplyTemplates(xsltTransformContextPtr ctxt, xmlNodePtr node,
 error:
     ctxt->mode = oldmode;
     ctxt->modeURI = oldmodeURI;
-    if (xpathParserCtxt != NULL)
-	xmlXPathFreeParserContext(xpathParserCtxt);
     if (res != NULL)
 	xmlXPathFreeObject(res);
     if (list != NULL)
@@ -2125,8 +2175,7 @@ void
 xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
     xmlChar *prop = NULL;
-    xmlXPathObjectPtr res = NULL, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
+    xmlXPathObjectPtr res = NULL;
     xmlNodePtr replacement, when;
     int doit = 1;
 
@@ -2149,7 +2198,8 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	goto error;
     }
     while (IS_XSLT_ELEM(replacement) && (IS_XSLT_NAME(replacement, "when"))) {
-
+	xmlXPathCompExprPtr comp;
+        /* TODO: build a precompiled block for when too ! */
 	when = replacement;
 	prop = xmlGetNsProp(when, (const xmlChar *)"test", XSLT_NAMESPACE);
 	if (prop == NULL) {
@@ -2162,21 +2212,15 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	     "xsl:when: test %s\n", prop);
 #endif
 
-	xpathParserCtxt = xmlXPathNewParserContext(prop, ctxt->xpathCtxt);
-	if (xpathParserCtxt == NULL)
+	comp = xmlXPathCompile(prop);
+	if (comp == NULL)
 	    goto error;
 	ctxt->xpathCtxt->node = node;
-	xmlXPathEvalExpr(xpathParserCtxt);
-	xmlXPathBooleanFunction(xpathParserCtxt, 1);
-	res = valuePop(xpathParserCtxt);
-	do {
-	    tmp = valuePop(xpathParserCtxt);
-	    if (tmp != NULL) {
-		xmlXPathFreeObject(tmp);
-	    }
-	} while (tmp != NULL);
-
+	res = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
+	xmlXPathFreeCompExpr(comp);
 	if (res != NULL) {
+	    if (res->type != XPATH_BOOLEAN)
+		res = xmlXPathConvertBoolean(res);
 	    if (res->type == XPATH_BOOLEAN)
 		doit = res->boolval;
 	    else {
@@ -2198,9 +2242,6 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    xsltFreeStackElemList(varsPop(ctxt));
 	    goto done;
 	}
-	if (xpathParserCtxt != NULL)
-	    xmlXPathFreeParserContext(xpathParserCtxt);
-	xpathParserCtxt = NULL;
 	if (prop != NULL)
 	    xmlFree(prop);
 	prop = NULL;
@@ -2227,8 +2268,6 @@ xsltChoose(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
 done:
 error:
-    if (xpathParserCtxt != NULL)
-	xmlXPathFreeParserContext(xpathParserCtxt);
     if (prop != NULL)
 	xmlFree(prop);
     if (res != NULL)
@@ -2247,8 +2286,7 @@ error:
 void
 xsltIf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
-    xmlXPathObjectPtr res = NULL, tmp;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
+    xmlXPathObjectPtr res = NULL;
     int doit = 1;
 
     if (comp == NULL) {
@@ -2260,27 +2298,22 @@ xsltIf(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     if (comp->test == NULL)
 	return;
+    if (comp->comp == NULL) {
+	comp->comp = xmlXPathCompile(comp->test);
+	if (comp->comp == NULL)
+	    return;
+    }
 
 #ifdef DEBUG_PROCESS
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltIf: test %s\n", comp->test);
 #endif
 
-    xpathParserCtxt = xmlXPathNewParserContext(comp->test, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	goto error;
     ctxt->xpathCtxt->node = node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    xmlXPathBooleanFunction(xpathParserCtxt, 1);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
-
+    res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     if (res != NULL) {
+	if (res->type != XPATH_BOOLEAN)
+	    res = xmlXPathConvertBoolean(res);
 	if (res->type == XPATH_BOOLEAN)
 	    doit = res->boolval;
 	else {
@@ -2303,8 +2336,6 @@ xsltIf(xsltTransformContextPtr ctxt, xmlNodePtr node,
     }
 
 error:
-    if (xpathParserCtxt != NULL)
-	xmlXPathFreeParserContext(xpathParserCtxt);
     if (res != NULL)
 	xmlXPathFreeObject(res);
 }
@@ -2321,10 +2352,9 @@ error:
 void
 xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	           xmlNodePtr inst, xsltStylePreCompPtr comp) {
-    xmlXPathObjectPtr res = NULL, tmp;
+    xmlXPathObjectPtr res = NULL;
     xmlNodePtr replacement;
     xmlNodeSetPtr list = NULL, oldlist;
-    xmlXPathParserContextPtr xpathParserCtxt = NULL;
     int i, oldProximityPosition, oldContextSize;
     /* xmlNodePtr oldInsert = ctxt->insert; */
     xmlNodePtr oldNode = ctxt->node;
@@ -2338,25 +2368,19 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
 
     if (comp->select == NULL)
 	return;
+    if (comp->comp == NULL) {
+	comp->comp = xmlXPathCompile(comp->select);
+	if (comp->comp == NULL)
+	    return;
+    }
 
 #ifdef DEBUG_PROCESS
     xsltGenericDebug(xsltGenericDebugContext,
 	 "xsltForEach: select %s\n", comp->select);
 #endif
 
-    xpathParserCtxt = xmlXPathNewParserContext(comp->select, ctxt->xpathCtxt);
-    if (xpathParserCtxt == NULL)
-	goto error;
     ctxt->xpathCtxt->node = node;
-    xmlXPathEvalExpr(xpathParserCtxt);
-    res = valuePop(xpathParserCtxt);
-    do {
-        tmp = valuePop(xpathParserCtxt);
-	if (tmp != NULL) {
-	    xmlXPathFreeObject(tmp);
-	}
-    } while (tmp != NULL);
-
+    res = xmlXPathCompiledEval(comp->comp, ctxt->xpathCtxt);
     if (res != NULL) {
 	if (res->type == XPATH_NODESET)
 	    list = res->nodesetval;
@@ -2403,8 +2427,6 @@ xsltForEach(xsltTransformContextPtr ctxt, xmlNodePtr node,
     ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
 
 error:
-    if (xpathParserCtxt != NULL)
-	xmlXPathFreeParserContext(xpathParserCtxt);
     if (res != NULL)
 	xmlXPathFreeObject(res);
 }
